unit Enh_Reg_Ini;
{ Enhanced Registry and IniFile components created by Amir Wald.
  Email: amirwald@writeme.com
  Last update: 22/09/99

  This unit implements two components.

  TEnhRegistry component inherits from TRegistry and adds only one published
  general method to read and write all kinds of data from/to the registry.
    procedure ReadWriteSth( const Name : string; var Value;
                            const defValue : variant; const VarType : TVarType;
                            const WriteIt : boolean );

    Name - the name of the data value to read/write.
    Value - the variable that the data will be read into/written from.
    defValue - the default value which will be assigned to Value if trying
               to read the value from the registry raises an exception.
    VarType - enumerated type that specifies the type of variable Value
              represents.
    WriteIt - True = Write value from Value to registry
              False = Read value from registry to Value


  TEnhIniFile component inherits from TIniFile and adds only one published
  general method to read and write all kinds of data from/to the ini file.
    procedure ReadWriteSth( const Section, Ident: string; var Value;
                            const defValue : variant; const VarType : TVarType;
                            const WriteIt : boolean );

    Section  - identifies the section in the file that contain the key.
    Ident - the Section, Ident of the key for which to set a value.
    Value - the variable that the data will be read into/written from.
    defValue - the default value which will be assigned to Value if trying
               to read the value from the ini file fails.
    VarType - enumerated type that specifies the type of variable Value
              represents.
    WriteIt - True = Write value from Value to registry
              False = Read value from registry to Value

}
interface
uses
  SysUtils, Registry, IniFiles;
type
  TVarType = ( vtBool, vtCurrency, vtDate, vtTime, vtDateTime,
                vtFloat, vtInteger, vtString, vtExpandString );

  TEnhRegistry = class( TRegistry )
  published
    procedure ReadWriteSth( const Name : string; var Value;
                            const defValue : variant; const VarType : TVarType;
                            const WriteIt : boolean );
  end;

  TEnhIniFile = class( TIniFile )
  published
    procedure ReadWriteSth( const Section, Ident: string; var Value;
                            const defValue : variant; const VarType : TVarType;
                            const WriteIt : boolean );
  end;



implementation

{#############################################################################}
{#############################################################################}
{#############################################################################}
procedure TEnhRegistry.ReadWriteSth;
  {===========================================================================}
  procedure ReadValue;
  begin
    try
      case VarType of
        vtBool :
          Boolean(Value):= ReadBool( Name );
        vtCurrency :
          Currency(Value):= ReadCurrency( Name );
        vtDate :
          TDateTime(Value):= ReadDate( Name );
        vtTime :
          TDateTime(Value):= ReadTime( Name );
        vtDateTime :
          TDateTime(Value):= ReadDateTime( Name );
        vtFloat :
          Double(Value):= ReadFloat( Name );
        vtInteger :
          Integer(Value):= ReadInteger( Name );
        vtString, vtExpandString :
          String(Value):= ReadString( Name );
      end
    except
      case VarType of
        vtBool :
          Boolean(Value):= Boolean(defValue);
        vtCurrency :
          Currency(Value):= Currency(defValue);
        vtDate, vtTime, vtDateTime :
          TDateTime(Value):= TDateTime(defValue);
        vtFloat :
          Double(Value):= Double(defValue);
        vtInteger :
          Integer(Value):= Integer(defValue);
        vtString, vtExpandString :
          String(Value):= String(defValue);
      end;
    end;
  end;
  {===========================================================================}
  procedure WriteValue;
  begin
    case VarType of
      vtBool :
        WriteBool( Name, Boolean(Value) );
      vtCurrency :
        WriteCurrency( Name, Currency(Value) );
      vtDate :
        WriteDate( Name, TDateTime(Value) );
      vtTime :
        WriteTime( Name, TDateTime(Value) );
      vtDateTime :
        WriteDateTime( Name, TDateTime(Value) );
      vtFloat :
        WriteFloat( Name, Double(Value) );
      vtInteger :
        WriteInteger( Name, Integer(Value) );
      vtString:
        WriteString( Name, String(Value) );
      vtExpandString :
        WriteExpandString( Name, String(Value) );
    end;
  end;
  {===========================================================================}
begin
  if WriteIt then
    WriteValue
  else
    ReadValue;
end;
{#############################################################################}
procedure TEnhIniFile.ReadWriteSth;
  {===========================================================================}
  procedure ReadValue;
  begin
    case VarType of
      vtBool :
        Boolean(Value):= ReadBool( Section, Ident, defValue );
      vtDate :
        TDateTime(Value):= ReadDate( Section, Ident, defValue );
      vtTime :
        TDateTime(Value):= ReadTime( Section, Ident, defValue );
      vtDateTime :
        TDateTime(Value):= ReadDateTime( Section, Ident, defValue );
      vtFloat :
        Double(Value):= ReadFloat( Section, Ident, defValue );
      vtInteger :
        Integer(Value):= ReadInteger( Section, Ident, defValue );
      vtString, vtExpandString :
        String(Value):= ReadString( Section, Ident, defValue );
    else
      raise Exception.Create( 'VarType is not recognized in TEnhIniFile.ReadValue' );
    end;
  end;
  {===========================================================================}
  procedure WriteValue;
  begin
    case VarType of
      vtBool :
        WriteBool( Section, Ident, Boolean(Value) );
      vtDate :
        WriteDate( Section, Ident, TDateTime(Value) );
      vtTime :
        WriteTime( Section, Ident, TDateTime(Value) );
      vtDateTime :
        WriteDateTime( Section, Ident, TDateTime(Value) );
      vtFloat :
        WriteFloat( Section, Ident, Double(Value) );
      vtInteger :
        WriteInteger( Section, Ident, Integer(Value) );
      vtString:
        WriteString( Section, Ident, String(Value) );
    else
      raise Exception.Create( 'VarType is not recognized in TEnhIniFile.WriteValue' );
    end;
  end;
  {===========================================================================}
begin
  if WriteIt then
    WriteValue
  else
    ReadValue;
end;
{#############################################################################}
{#############################################################################}
{#############################################################################}
end.
